use smol::fs::File;
use smol::io::{AsyncBufReadExt, BufReader};

use crate::models::station_measurements::StationMeasurements;
use crate::utils::{hash, parse};
use std::collections::HashMap;
use std::time::Instant;

const DEFAULT_HASHMAP_LENGTH: usize = 10000;

pub fn run() {
    let now = Instant::now();
    let mut stations: HashMap<usize, (String, StationMeasurements)> =
        HashMap::with_capacity(DEFAULT_HASHMAP_LENGTH);

    smol::block_on(async {
        let mut file = File::open("../../../measurements.txt")
            .await
            .expect("Could not read file");
        let mut reader = BufReader::new(&mut file);
        let mut line = Vec::with_capacity(108);
        loop {
            let line_len = reader
                .read_until(b'\n', &mut line)
                .await
                .expect("could not read bytes");
            if line_len == 0 {
                break;
            }
            let (station, temp) = line.rsplit_once(|&byte| byte == b';').unwrap();
            let hash = hash::bytes(station);
            let station = unsafe { std::str::from_utf8_unchecked(station) };
            let temp = parse::temp(temp.split_last().unwrap().1);
            let measurements_option = stations.get_mut(&hash);
            if let Some((_, measurements)) = measurements_option {
                measurements.update(temp);
            } else {
                let measurements = StationMeasurements {
                    min: temp,
                    max: temp,
                    count: 1,
                    sum: temp,
                };
                stations.insert(hash, (station.to_string(), measurements));
            }
            line.clear();
        }
        let mut stations: Vec<String> = stations
            .iter()
            .map(|(_, (station, measurements))| {
                let measurements = measurements.to_string();
                format!("{station}={measurements}")
            })
            .collect();
        stations.sort();
        let stations = stations.join(",");
        println!("{{{stations}}}");
        println!("Time={} ms", now.elapsed().as_millis());
    })
}
